#include "INA219_DRV.h"
#include "..\IIC\IIC.h"

unsigned short INA219_ReadReg(unsigned char RegBase)
{
	unsigned short RegValue = 0;

	IIC_Start();
	IIC_SendOneByte(INA219_WR_Operation);
	if(!IIC_RecvACK_Error())
	{
		IIC_SendOneByte(RegBase);
		if(!IIC_RecvACK_Error())
		{
			//IIC_Stop();
			IIC_Start();
			IIC_SendOneByte(INA219_RD_Operation);
			if(!IIC_RecvACK_Error())
			{
				RegValue = IIC_RecvOneByte();
				IIC_SendACK(ACK);
				RegValue = RegValue << 8; 
				RegValue |= IIC_RecvOneByte();
				IIC_SendACK(NACK);
			}
		}
	}
	IIC_Stop();

	return RegValue;
}

bit INA219_WriteReg(unsigned char RegBase, unsigned short RegValue)
{
	bit WrRegError = 1;

	IIC_Start();
	IIC_SendOneByte(INA219_WR_Operation);
	if(!IIC_RecvACK_Error())
	{
		IIC_SendOneByte(RegBase);
		if(!IIC_RecvACK_Error())
		{
			IIC_SendOneByte((RegValue & 0xFF00) >> 8);
			IIC_RecvACK_Error();
			IIC_SendOneByte(RegValue & 0x00FF);
			WrRegError = IIC_RecvACK_Error();
		}
	}
	IIC_Stop();

	return !WrRegError;
}

void INA219_Init(void)
{
	INA219_WriteReg(0x00, 0x2667);//32V FSR, GAIN = 1, 16 samples, Shunt and Bus, Continuous.
	INA219_WriteReg(0x05, 0x5477);//Use INA219 EVM software to calculate a full-scale calibration value (initial calbration), and then compute the corrected full-scale calibration value based on measured current (second calbration). 
}

unsigned short INA219_GetBusVolt(void)
{
	unsigned short BusVoltRegVal = INA219_ReadReg(0x02);
	if(BusVoltRegVal & 0x0001)								//Overflow
		return 0x0FA0;
	else
		return (BusVoltRegVal & 0xFFF8) >> 1;		//((BusVoltRegVal & 0xFFF8) >> 3)*4mV
}

unsigned long INA219_GetPower(void)
{
	unsigned short PowerRegVal = INA219_ReadReg(0x03);
	return (PowerRegVal >> 1)*5;							//2.5mW/bit
}

unsigned short INA219_GetCurrent(void)
{
	unsigned short CurrentRegVal = INA219_ReadReg(0x04);
	return CurrentRegVal >> 3;								//0.125mA/bit
}