#include "LCD1602\LCD1602.h"
#include "INA219_DRV\INA219_DRV.h"
#include "common\common.h"

#define TH0_VALUE 0x4B
#define TL0_VALUE 0xFF										//50ms per cycle.

sbit BtnRefreshMode = P3^2;
bit RefreshRate = 0;											//0: 0.8s, 1: 0.5s
bit RefreshFlag = 0;

void RefreshData(void)
{
	unsigned short BusVolt, Current;
	unsigned long Power;
	unsigned short OffsetCurrent;
	//unsigned long OffsetPower;
		
	BusVolt = INA219_GetBusVolt();
	PrintChar(0, 0, BusVolt / 10000 + 48);
	PrintChar(1, 0, (BusVolt % 10000) / 1000 + 48);
	PrintChar(3, 0, (BusVolt % 1000) / 100 + 48);
	PrintChar(4, 0, (BusVolt % 100) / 10 + 48);
	PrintChar(5, 0, BusVolt % 10 + 48);
		
	if(BusVolt < 20000)
		OffsetCurrent = (BusVolt >> 9) + 2;			//Gather statistics of the measured current under various voltage conditions when no load is connected to the output port. Use Excel to do curve fitting.
	else
		OffsetCurrent = (BusVolt >> 9) + 4;
	Current = INA219_GetCurrent();
	if(Current > OffsetCurrent)
		Current -= OffsetCurrent;
	else
		Current = 0;
	PrintChar(10, 0, Current / 1000 + 48);
	PrintChar(12, 0, (Current % 1000) / 100 + 48);
	PrintChar(13, 0, (Current % 100) / 10 + 48);
	PrintChar(14, 0, Current % 10 + 48);
		
//	OffsetPower = ((BusVolt >> 3) * OffsetCurrent) / 125 + 6;
//	Power = INA219_GetPower();
//	if(Power > OffsetPower)
//		Power -= OffsetPower;
//	else
//		Power = 0;
	Power = (unsigned long)BusVolt * (unsigned long)Current;
	PrintChar(0, 1, Power / 10000000 + 48);
	PrintChar(1, 1, (Power % 10000000) / 1000000 + 48);
	PrintChar(3, 1, (Power % 1000000) / 100000 + 48);
	PrintChar(4, 1, (Power % 100000) / 10000 + 48);
	PrintChar(5, 1, (Power % 10000) /1000 + 48);
}

int main(void)
{
	INA219_Init();
	LCD_Init();
	
	TMOD &= 0xF0;
	TMOD |= 0x01;										//Timer 0 works in Mode 1 (16-bit Timer).

	TH0 = TH0_VALUE;
	TL0 = TL0_VALUE;
	
	PrintChar(2, 0, '.');
	PrintChar(6, 0, 'V');
	PrintChar(11, 0, '.');
	PrintChar(15, 0, 'A');
	PrintChar(2, 1, '.');
	PrintChar(6, 1, 'W');
	PrintStr(10, 1, "R:0.8s");
	RefreshData();
	
	EA = 1;													//Global Interrupt Enable
	ET0 = 1;												//Timer 0 Interrupt Enable
	TR0 = 1;												//Enable Timer 0.
	
	while(1)
	{
		if(RefreshFlag)
		{
			RefreshData();
			RefreshFlag = 0;
		}
		
		if(BtnRefreshMode == 0)				//If the refreshing-mode button is pressed
		{
			Delay_us(5000);
			if(BtnRefreshMode == 1)
			{
				Delay_us(5000);						//Debouncing
				if(BtnRefreshMode == 1)
				{
					RefreshRate= !RefreshRate;
					if(RefreshRate)
						PrintChar(14, 1, '5');
					else
						PrintChar(14, 1, '8');
				}
			}
		}
	}

	return 0;
}

void Timer0ISR(void) interrupt 1 using 2
{
	static unsigned char Counter = 0;

	TH0 = TH0_VALUE;
	TL0 = TL0_VALUE;
	
	if(Counter < 15-6*RefreshRate)	//Approximately 0.8s for slow-refreshing mode and 0.5s for fast-refreshing mode
		Counter++;
	else
	{
		Counter = 0;
		RefreshFlag = 1;
	}
}